<!DOCTYPE html>
<html>
<head>
<title>Welcome to {{NAME}}!</title>
<style>
    body {
        width: 35em;
        margin: 0 auto;
        font-family: Tahoma, Verdana, Arial, sans-serif;
    }
</style>
</head>
<body>
<h1>Welcome to {{NAME}}!</h1>
<p>The {{NAME}} version {{VERSION}} server was configured on this host at {{DATETIME}}.</p>



<p>Here are the pre-defined public targets supplied by lorax:
    <ul>
    <li><a href="status">{{NAME}} status</a></li>
    <li><a href="healthcheck">{{NAME}} healthcheck</a></li>
    <li><a href="trees/families.json">List of defined gene families</a></li>
    </ul>
</p><br/>

<p>If you have run the test suite that creates the "aspartic peptidases" family,
    the following public targets wil be defined:
    <ul>
    <li><a href="trees/aspartic_peptidases/alignment">alignment</a></li>
    <li><a href="trees/aspartic_peptidases/hmmalign/run_log.txt">alignment run log</a></li>
    <li><a href="trees/aspartic_peptidases/FastTree/tree.nwk">tree (Newick format)</a></li>
    <li><a href="trees/aspartic_peptidases/FastTree/run_log.txt">FastTree run log</a></li>
    </ul>
</p><br/>

<p>The following targets require authentication:
    <ul>
    <li><a href="badtarget">Throw an intentional server error</a></li>
    <li><a href="log.txt">Access/error log file</a></li>
    <li><a href="environment">Environmental variables</a></li>
    <li><a href="rq">Status of queues (with errors)</a></li>
    <li><a href="supervisord">Control processes (via supervisord)</a></li>
    <li><a href="prometheus">Prometheus stats monitoring</a></li>
    </ul>
</p><br/>

<p>For online documentation and support please refer to the
    <a href="{{PROJECT_HOME}}">{{NAME}} home site</a>.<br/>
   <em>Thank you for using {{NAME}}.</em></p>
</body>
</html>
